def get_pronoun(tag):
	#Pp1-sn--------s
	return tag[2], tag[4], tag[3]

def get_verb(tag):
	if len(tag) < 6:
		number = ""
	else:
		number = tag[5]
	return tag[4], number

def get_noun(tag):
	return '3', tag[3], tag[2]

def get_adjective(tag):
	return "-", tag[4], tag[3]

def get_numeral(tag):
	return "-", tag[3], tag[2]

def get_person_number_gender(spn):
	person = ""
	number = ""
	gender = ""
	if spn[0] == "3":
		person = "a 3-a"
	else:
		person = spn[0]
	if spn[1] == "s":
		number = "singular"
	else:
		number = "plural"
	if spn[2] == "f":
		gender = "feminin"
	else:
		gender = "masculin"
	return person, number, gender


from rb.parser.spacy_parser import SpacyParser

from spacy.tokens.doc import Doc

from rb.core.lang import Lang
from rb.core.text_element import TextElement
from rb.core.word import Word
from rb.core.text_element_type import TextElementType

from typing import Tuple, List
Dependency = Tuple[Word, Word, str]
Dependencies = List[Dependency]

class Sentence(TextElement):


    def __init__(self, lang: Lang, text: str,
                 depth: int = TextElementType.SENT.value,
                 container: TextElement = None):

        TextElement.__init__(self, lang=lang, text=text,
                             depth=depth, container=container)
        doc = SpacyParser.get_instance().parse(text, lang.value)

        for token in doc:
            word = Word(lang, token, container=self)
            self.components.append(word)
        for word, token in zip(self.components, doc):
            word.head = self.components[token.head.i]
            if word.head is not word:
                word.head.children.append(word)
        self.ents = doc.ents
        self.root = self.components[list(doc.sents)[0].root.i]

    def get_dependencies(self) -> Dependencies:
        return [(word.head, word, word.dep) 
                for word in self.components 
                if word.head != word]
            
    def get_sentences(self) -> List["Sentence"]:
        return [self]


from enum import Enum
from rb.complexity.surface.factory import create as surface
from rb.complexity.syntax.factory import create as syntax
from rb.complexity.morphology.factory import create as morphology
from rb.complexity.word.factory import create as word
from functools import partial

class IndexCategory(Enum):
    SURFACE = partial(surface)
    MORPHOLOGY = partial(morphology)
    SYNTAX = partial(syntax)
    WORD = partial(word)



from rb.parser.spacy_parser import SpacyParser

from rb.core.lang import Lang
from rb.core.text_element import TextElement
from rb.core.sentence import Sentence
from rb.core.text_element_type import TextElementType

class Block(TextElement):


    def __init__(self, lang: Lang, text: str,
                 depth: int = TextElementType.BLOCK.value,
                 container: TextElement = None):

        TextElement.__init__(self, lang=lang, text=text,
                             depth=depth, container=container)
        for sentence in SpacyParser.get_instance().tokenize_sentences(text):
            self.components.append(Sentence(lang, sentence, container=self))



from typing import List

from rb.core.lang import Lang
from rb.complexity.measure_function import MeasureFunction
from rb.core.text_element_type import TextElementType
from rb.complexity.measure_function import MeasureFunction
# create all indices
# dependencies need to be putted in function because otherwise circular dependencies happens
def create(lang: Lang) -> List["ComplexityIndex"]:
    from rb.complexity.word.wd_len import WdLen
    from rb.complexity.word.wd_diff_lemma import WdDiffLemma
    from rb.complexity.word.no_repetitions import NoRepetitions
    from rb.complexity.word.name_entity_enum import NamedEntityONEnum
    from rb.complexity.word.name_entity import NoNamedEntity

    indices = []
    indices.append(WdLen(lang, TextElementType.WORD.value, MeasureFunction.AVG))
    indices.append(WdDiffLemma(lang, TextElementType.WORD.value, MeasureFunction.AVG))
    indices.append(NoRepetitions(lang, window_size=8, reduce_depth=TextElementType.SENT.value, 
                                    reduce_function=MeasureFunction.AVG))
    indices.append(NoNamedEntity(lang, named_ent_type=NamedEntityONEnum.NORP, reduce_depth=TextElementType.WORD.value, 
                                    reduce_function=MeasureFunction.AVG))
    #indices = [DepIndex(lang, dep, TextElementType.BLOCK, MeasureFunction.AVG) for dep in DepEnum]
    #indices += [DepIndex(lang, dep, TextElementType.BLOCK, MeasureFunction.STDEV) for dep in DepEnum]
    return indices


from typing import List

from rb.core.lang import Lang
from rb.complexity.measure_function import MeasureFunction
from rb.core.text_element_type import TextElementType
from rb.complexity.measure_function import MeasureFunction
# create all indices
# dependencies need to be putted in function because otherwise circular dependencies happens
def create(lang: Lang) -> List["ComplexityIndex"]:
    from rb.complexity.syntax.dep import DepIndex
    from rb.complexity.syntax.dep_enum import DepEnum
    from rb.complexity.syntax.parse_dep_tree import ParseDepTree

    indices = []
    indices.append(ParseDepTree(lang, TextElementType.SENT.value, MeasureFunction.AVG))
    #indices = [DepIndex(lang, dep, TextElementType.BLOCK, MeasureFunction.AVG) for dep in DepEnum]
    #indices += [DepIndex(lang, dep, TextElementType.BLOCK, MeasureFunction.STDEV) for dep in DepEnum]
    return indices



from typing import Callable, List, Tuple, Union

from nltk.corpus import wordnet as wn
from nltk.corpus.reader.wordnet import Synset
from rb.core.lang import Lang
from rb.core.word import Word

from rb.parser.spacy_parser import SpacyParser

lang_dict = {
    Lang.EN: 'eng',
    Lang.NL: 'nld',
    Lang.FR: 'fra',
    Lang.RO: 'ron',
    Lang.IT: 'ita'
}

def compute_similarity(a: str, b: str, lang: Lang, sim: Callable[[Synset, Synset], float]) -> float:
    if lang not in lang_dict:
        return 0
    lang = lang_dict[lang]
    return min([
        sim(syn_a, syn_b)
        for syn_a in wn.synsets(a, lang=lang)
        for syn_b in wn.synsets(b, lang=lang)],
        default=0)

def path_similarity(a: str, b: str, lang: Lang) -> float:
    return compute_similarity(a, b, lang, wn.path_similarity)

def leacock_chodorow_similarity(a: str, b: str, lang: Lang) -> float:
    return compute_similarity(a, b, lang, wn.lch_similarity)

def wu_palmer_similarity(a: str, b: str, lang: Lang) -> float:
    return compute_similarity(a, b, lang, wn.wup_similarity)

def get_synonyms(word: Union[str, Word], lang: Lang = None, pos: str = None) -> List[str]:
    if isinstance(word, Word):
        pos = word.pos.to_wordnet()
        lang = word.lang
        word = word.lemma
    if lang not in lang_dict:
        return []
    return list({other
        for ss in wn.synsets(word, pos=pos, lang=lang_dict[lang]) 
        for other in ss.lemma_names(lang=lang_dict[lang]) 
        if SpacyParser.get_instance().is_dict_word(other, lang)})

def get_hypernyms(word: Union[str, Word], lang: Lang = None, pos: str = None) -> List[str]:
    if isinstance(word, Word):
        pos = word.pos.to_wordnet()
        lang = word.lang
        word = word.lemma
    if lang not in lang_dict:
        return []
    return list({other
        for ss in wn.synsets(word, pos=pos, lang=lang_dict[lang]) 
        for parent in ss.hypernyms() 
        for other in parent.lemma_names(lang=lang_dict[lang]) 
        if SpacyParser.get_instance().is_dict_word(other, lang)})
        

if __name__ == "__main__":
    print(path_similarity('hond', 'kat', 'nl'))
    print(get_hypernyms('animal', lang_dict[Lang.RO]))



from enum import Enum
from typing import Union

import numpy as np
from numpy.linalg import norm

from rb.core.lang import Lang
from rb.core.text_element import TextElement
from rb.core.word import Word


class VectorModelType(Enum):
    LSA = 0,
    LDA = 1,
    WORD2VEC = 2,
    FASTTEXT = 3,
    GLOVE = 4


class VectorModel:
    def __init__(self, type: VectorModelType, name: str, lang: Lang, size: int = 300):
        self.lang = lang
        self.type = type
        self.name = name
        self.size = size
        self.vectors = {}

    def get_vector(self, elem: Union[str, TextElement]) -> np.array:
        if isinstance(elem, str):
            return self.vectors[elem]
        if not self in elem.vectors:
            if isinstance(elem, Word):
                if elem.text in self.vectors:
                    elem.vectors[self] = self.vectors[elem.text]
                elif elem.lemma in self.vectors:
                    elem.vectors[self] = self.vectors[elem.lemma]
                else:
                    return None
            else:
                elem.vectors[self] = np.sum([child for child in elem.components])
        return elem.vectors[self]
            
    
    def similarity(self, a: Union[TextElement, np.array], b: Union[TextElement, np.array]) -> float:
        if isinstance(a, TextElement) and isinstance(b, TextElement) and a == b:
            return 1.0
        if isinstance(a, TextElement):
            a = self.get_vector(a)
        if isinstance(b, TextElement):
            b = self.get_vector(b)
        if a is None or b is None:
            return 0.0
        return np.dot(a, b) / (norm(a) * norm(b))



import json
import os
import sys
import zipfile
from typing import List, Union

import wget
from rb.core.lang import Lang
from rb.utils.rblogger import Logger

LINKS = {
    Lang.EN: {
        'models': {
            'coca': {
                'link': "https://nextcloud.readerbench.com/index.php/s/TxamWx6Er9G2wDo/download",
                'version': "https://nextcloud.readerbench.com/index.php/s/npzePeQrpHLE8N6/download"
            },
            'tasa': {
                'link': "https://nextcloud.readerbench.com/index.php/s/CSqbbkxm4En2KKE/download",
                'version': "https://nextcloud.readerbench.com/index.php/s/cXikjDcESTtsJDC/download"
            },
        },
        'spacy': {}

    },
    Lang.RO: {
        'models': {
            'diacritics': "https://nextcloud.readerbench.com/index.php/s/pfC25G64JgxcfZS/download",
            'readme': "https://nextcloud.readerbench.com/index.php/s/g9etLBeSTmKxjM8/download",
        },
        'spacy': {
            'ro_ud_ft_ner': {
                'link': "https://nextcloud.readerbench.com/index.php/s/rCxP6gobmHi8KKe/download",
                'version': "https://nextcloud.readerbench.com/index.php/s/egSNXJfptiqSx8k/download"
            }
        },
        'wordnet': "https://nextcloud.readerbench.com/index.php/s/7tDka2CSGYeJqgC/download"
    },
    Lang.RU: {
        'spacy': {
            'ru_ud_ft': "https://nextcloud.readerbench.com/index.php/s/nbErkdgbxmgiRCG/download"
        }
    }
}

logger = Logger.get_logger()

def download_folder(link: str, destination: str):
    os.makedirs(destination, exist_ok=True)     
    filename = wget.download(link, out=destination, bar=wget.bar_thermometer)
    logger.info('Downloaded {}'.format(filename))
    if zipfile.is_zipfile(filename):
        logger.info('Extracting files from {}'.format(filename))
        with zipfile.ZipFile(filename,"r") as zip_ref:
            zip_ref.extractall(destination)
        os.remove(filename)
    

def download_model(lang: Lang, name: Union[str, List[str]]) -> bool:
    if isinstance(name, str):
        name = ['models', name]
    if not lang in LINKS:
        logger.info('{} not supported.'.format(lang))
        return False
    path = "/".join(name)
    root = LINKS[lang]
    for key in name:
        if key not in root:
            logger.info('Remote path not found {} ({}).'.format(path, key))
            return False
        root = root[key]
    logger.info("Downloading model {} for {} ...".format(path, lang.value))
    link = root['link'] if isinstance(root, dict) else root
    folder = "resources/{}/{}".format(lang.value, "/".join(name[:-1]))
    download_folder(link, folder)
    return True

def download_spacy_model(lang: Lang, name: str) -> bool:
    return download_model(lang, ['spacy', name])

def download_wordnet(lang: Lang, folder: str) -> bool:
    if lang not in LINKS:
        logger.info('{} not supported.'.format(lang))
        return False
    if 'wordnet' not in LINKS[lang]:
        logger.info('No WordNet found')
        return False
    link = LINKS[lang]['wordnet']
    download_folder(link, folder)
        
def check_spacy_version(lang: Lang, name: str) -> bool:
    return check_version(lang, ['spacy', name])

def check_version(lang: Lang, name: Union[str, List[str]]) -> bool:
    if isinstance(name, str):
        name = ['models', name]
    path = "/".join(name)
    folder = "resources/{}/{}".format(lang.value, path)
    try:
        local_version = read_version(folder + "/version.txt")
    except:
        logger.info('Local model {} for {} not found.'.format(path, lang))
        return True
        
    if not lang in LINKS:
        logger.info('{} not supported.'.format(lang))
        return False
    root = LINKS[lang]
    for key in name:
        if key not in root:
            logger.info('Remote path not found {} ({}).'.format(path, key))
            return False
        root = root[key]
    if isinstance(root, dict):
        filename = wget.download(root['version'], out="resources/", bar=wget.bar_thermometer)
        try:
            remote_version = read_version(filename)
        except:
            logger.info('Error reading remote version for {} ({})'.format(path, lang))
            return False
        return newer_version(remote_version, local_version)
    else:
        return True
    
def read_version(filename: str) -> str:
    with open(filename, "r") as f:
        return f.readline()

def newer_version(remote_version: str, local_version: str) -> bool:
    remote_version = remote_version.split(".")
    local_version = local_version.split(".")
    for a, b in zip(remote_version, local_version):
        if int(a) > int(b):
            return True
        if int(a) < int(b):
            return False
    return False   
    
        

if __name__ == "__main__":
    download_model(Lang.EN, 'coca')




from rb.core.lang import Lang
from rb.core.text_element import TextElement
from rb.complexity.index_category import IndexCategory
from rb.utils.rblogger import Logger 
from typing import List, Iterable, Callable
from rb.core.text_element_type import TextElementType
from rb.complexity.measure_function import MeasureFunction
logger = Logger.get_logger()

class ComplexityIndex():
    """General class for any complexity index
    
    Attributes
    ----------
    lang : Lang
        language where the index is applied
    category : IndexCategory
        type of the index e.g. SURFACE, SYNTAX
    abbr : str
        short string describing the index
    reduce_depth : TextElementType
        the depth (in the document) where the reduce_function is applied
        the index is applied recursivley on all the above element types from the document tree
    reduce_function : Callable[[List], float]
        a function to summarize the results of the index (average or standard deviation)

    Methods
    -------
    process(element: TextElement)
        computes the index, overwritten for each index
    
    __repr__()
        detailed string representation of the index, should overwritten by each index
    """

    def __init__(self, lang: Lang, category: IndexCategory, abbr: str, reduce_depth: int, reduce_function: MeasureFunction):
        self.lang = lang
        self.category = category
        self.abbr = abbr
        self.reduce_function = reduce_function
        self.reduce_depth = reduce_depth

    # overwritten by each index
    def process(self, element: TextElement) -> float:
        pass

    # overwritten by each index 
    def __repr__(self):
        return self.abbr

# computed indices and saves for each TextElement in indices dictionary
def compute_indices(element: TextElement):
    logger.info('Starting computing all indices for {0} type element'.format(type(element).__name__))
    for cat in IndexCategory:
        for index in cat.value(element.lang):
            index.process(element)

    

    


from rb.core.lang import Lang
from rb.core.text_element import TextElement
from rb.core.block import Block
from rb.core.text_element_type import TextElementType

class Document(TextElement):
    

    def __init__(self, lang: Lang, text: str,
                 depth: int = TextElementType.DOC.value,
                 container: TextElement = None):
        TextElement.__init__(self, lang=lang, text=text,
                             depth=depth, container=container)
        for block in text.split("\n"):
            self.components.append(Block(lang=lang, text=block.strip(),
                                         container=self))



from typing import List
from rb.core.lang import Lang
from rb.core.text_element_type import TextElementType

def create(lang: Lang) -> List["ComplexityIndex"]:
    from rb.complexity.surface.no_words import NoWordsIndex
    from rb.complexity.surface.no_unq_words import NoUniqueWordsIndex
    from rb.complexity.surface.no_commas import NoCommas
    from rb.complexity.surface.no_punctuations import NoPunctuations
    from rb.complexity.surface.no_sentences import NoSentences
    from rb.complexity.surface.wd_entropy import WdEntropy
    from rb.complexity.surface.ch_entropy import ChEntropy
    from rb.complexity.surface.ch_ngram_entropy import ChNgramEntropyEnum, ChNgramEntropy
    from rb.complexity.surface.no_cacophonies import NoCacophonies
    from rb.complexity.surface.no_common_errors import NoCommonErrors

    from rb.complexity.syntax.dep import DepIndex
    from rb.complexity.syntax.dep_enum import DepEnum

    from rb.complexity.measure_function import MeasureFunction

    indices = []
    indices.append(NoWordsIndex(lang, TextElementType.SENT.value, MeasureFunction.AVG))
    indices.append(NoUniqueWordsIndex(lang, TextElementType.SENT.value, MeasureFunction.AVG))
    indices.append(NoSentences(lang, TextElementType.SENT.value, MeasureFunction.AVG))
    indices.append(NoPunctuations(lang, TextElementType.SENT.value, MeasureFunction.AVG))
    indices.append(NoCommas(lang, TextElementType.SENT.value, MeasureFunction.AVG))
    indices.append(WdEntropy(lang, TextElementType.SENT.value, MeasureFunction.AVG))
    indices.append(ChEntropy(lang, TextElementType.WORD.value, MeasureFunction.AVG))
    indices.append(ChNgramEntropy(lang, ChNgramEntropyEnum.TWO, TextElementType.WORD.value, MeasureFunction.AVG))
    indices.append(NoCacophonies(Lang.RO, TextElementType.SENT.value, MeasureFunction.AVG))
    indices.append(NoCommonErrors(Lang.RO, TextElementType.SENT.value, MeasureFunction.AVG))
    # for dep_type in DepEnum:
    #     indices.append(DepIndex(lang, dep_type, TextElementType.SENT.value, MeasureFunction.AVG))
    
    return indices

    


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='rbpy-rb',
    version='0.2.7',
    author='Woodcarver',
    author_email='batpepastrama@gmail.com',
    description='ReaderBench library written in python',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://git.readerbench.com/ReaderBench/Readerbench-python',
    data_files=[('.json', ['rb/ro_corrections/exceptions.json', 'rb/ro_corrections/lexicons.json'])],
    packages=setuptools.find_packages(),
    dependency_links=['https://github.com/explosion/spacy-models/releases/download/en_core_web_sm-2.1.0/en_core_web_sm-2.1.0.tar.gz'],
    install_requires=[
          'wheel',
          'spacy==2.1.3',
          'nltk',
          'gensim',
          'numpy',
          'wget',
          'pymorphy2',
          'neuralcoref'
      ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent"
    ],
)
